/*
 * W. John Thrasher
 * CAP4601
 * Research Project
 * Class Schedule definition
 */

#include <iostream> 
#include <array>
 
#include "schedule.h"
 
typedef std::pair< std::string, int > team;

 
//Schedule constructor
Schedule::Schedule(std::vector< std::pair< std::string, int > > lg0, std::vector< std::pair< std::string, int > > lg1,
			std::vector< float > d) : league0(lg0), league1(lg1), distances(d)
{
	totalDistance = 0;
}

//calculates the distances traveled by awayGroup in a given homestand
float Schedule::CalcDistance(std::vector< std::pair< std::string, int > > homeGroup, std::vector< std::pair< std::string, int > > awayGroup, 	std::vector<float> distances) {
	float distance = 0;
	size_t currentIndex = 0;
	
	//only works if groups are same size
	if (homeGroup.size() != awayGroup.size())
		return -1.0f;
	//total distance for an away std will always be twice distance from lowest to highest index
	for (size_t i = 0; i < awayGroup.size(); ++i) {
		size_t lowestIndex = awayGroup[i].second;
		size_t highestIndex = awayGroup[i].second;
		//find highest and lowest index		
		for (size_t j = 0; j < homeGroup.size(); ++j) {
			currentIndex = homeGroup[j].second;
			if (currentIndex < lowestIndex)
				lowestIndex = currentIndex;
			else if (currentIndex > highestIndex)
				highestIndex = currentIndex;
		}
		//add double the distance from lowest to highest index to total
		for (size_t k = lowestIndex; k < highestIndex; ++k) {

			distance += (2 * distances[k]);
		}
	}

	return distance;
} //CalcDistance


//Main scheduling function
void Schedule::CreateSchedule()  {
	//reset total distance
	totalDistance = 0;
	groups0 = Split(league0);
	groups1 = Split(league1);

	size_t numGroups = groups0.size();
	
	for (size_t i = 0; i < numGroups; ++i) {
		for (size_t j = 0; j < numGroups; ++j) {		
			size_t g1Index = i + j;
			if (g1Index >= numGroups)
				g1Index -= numGroups;			
			//Once for group 0 homestand
			totalDistance += CalcDistance(groups0[i], groups1[g1Index], distances);
			//Once for group 1 homestand
			totalDistance += CalcDistance(groups1[g1Index], groups0[i], distances); 
		}
	}	
} //Schedule()

//Splits std into groups of 3 or less
std::vector< std::vector< std::pair< std::string, int > > > Schedule::Split(std::vector< std::pair< std::string, int > > league) {
	
	std::vector< std::vector< std::pair< std::string, int > > > returnVec;
	
	size_t size{league.size()};
	
	for (size_t i = 0; i < size; ++i) {
		std::vector< std::pair< std::string, int > > current;		
		for (size_t j = 0; j < GROUP_SIZE && ((i + j) < size); ++j) {
			current.push_back(league[i + j]);		
		}
		
		i += GROUP_SIZE - 1;
				
		returnVec.push_back(current);
		current.clear();
	}
	
	return returnVec;
} //Split()


//Print out results
void Schedule::Print() {
		
	size_t numGroups = groups0.size();
	std::array< std::array< size_t, 3 >, 3 > groupOrder{ { 
		{ {0, 1, 2} }, 
		{ {1, 2, 0} }, 
		{ {2, 0, 1} } } };
	//loop through for league 0 schedule
	for (size_t i = 0; i < numGroups; ++i) {
		//loop through each individual group
		for (size_t j = 0; j < groups0[i].size(); ++j) {
			std::cout << groups0[i][j].first << " | ";
			PrintHomeFirst(groups1, groupOrder[j], i);
			
		}
	
	}
	
	//loop through for league 1 schedule
	for (size_t i = 0; i < numGroups; ++i) {
	
		//loop through each individual group
		for (size_t j = 0; j < groups1[i].size(); ++j) {
			
			std::cout << groups1[i][j].first << " | ";
			PrintAwayFirst(groups0, groupOrder[j], i);
			
		}
	
	}


	std::cout << "Total Distances:  " << totalDistance << "km\n";

} //Print()

//Print out results for an individual team in the league that has the first homestand
void Schedule::PrintHomeFirst(std::vector< std::vector< team > > otherGroups, std::array< size_t , 3 >  order, size_t position) {
	for (size_t i = position; i < otherGroups.size(); ++i) {
		for (auto j : order) {
			if (j < otherGroups[i].size())
				std::cout << "h_" << otherGroups[i][j].first << " ";
		}
		for (auto j : order) {
			if (j < otherGroups[i].size())
				std::cout << "a_" << otherGroups[i][j].first << " ";
		}
	}
	
	for (size_t i = 0; i < position; ++i) {
		
		for (auto j : order) {
			if (j < otherGroups[i].size())
				std::cout << "h_" << otherGroups[i][j].first << " ";
		}
		
		for (auto j : order) {
			if (j < otherGroups[i].size())
				std::cout << "a_" << otherGroups[i][j].first << " ";
		}
	}
	
	std::cout << " | \n";
} //PrintHomeFirst()

//Print out results for an individual team in the league that has the second homestand
void Schedule::PrintAwayFirst(std::vector< std::vector< team > > otherGroups, std::array< size_t, 3 > order, size_t position) {

	for (size_t i = position; i < otherGroups.size(); ++i) {
		
		for (auto j : order) {
			if (j < otherGroups[i].size())
				std::cout << "a_" << otherGroups[i][j].first << " ";
		}
		
		for (auto j : order) {
			if (j < otherGroups[i].size())
				std::cout << "h_" << otherGroups[i][j].first << " ";
		}
	}
	
	for (size_t i = 0; i < position; ++i) {
		
		for (auto j : order) {
			if (j < otherGroups[i].size())
				std::cout << "a_" << otherGroups[i][j].first << " ";
		}
		
		for (auto j : order) {
			if (j < otherGroups[i].size())
				std::cout << "h_" << otherGroups[i][j].first << " ";
		}
	}
	std::cout << " | \n";
} //PrintAwayFirst()